//*************************************************
// Utilizzo di un tastierino a matrice 4x4
// www.settorezero.com
//
// modulo: main.c
// autore: Bernardo Giovanni
// data: 13/06/10
// picmicro: PIC16F877A
// clock: 20MHz
//
//*************************************************

#define  XTAL_FREQ 20MHZ // questo  utilizzato dalle routine di ritardo contenute in Delay.C
#include <pic.h> // contiene i nomi mnemonici di registri e porte
// Fuses di configurazione
__CONFIG (HS & WDTDIS & PWRTEN & BORDIS & LVPDIS & DUNPROT & WRTEN & DEBUGDIS & UNPROTECT);

#include "delay.c" // routine per ritardi
#include "settings.h"
#include "lcd.c"

void main(void)
	{
	settings();
	LCDBL=1;
	LCD_INIT(); // inizializzo l'LCD
	DelayMs(100); // piccolo ritardo iniziale
	LCD_CLEAR(); // ripulisco il display
	
	LCD_GOTO(1,1); // posiziono il cursore sull'LCD: riga1, colonna1
	LCD_PUTS("Numeric   Keypad");
	LCD_GOTO(2,1); // riga2, colonna1
	LCD_PUTS("settorezero.com");
	
	DelayMs(250);
	DelayMs(250);
	DelayMs(250);
	
	LCD_CLEAR();
	
	keyok=0;
	
	while(1)
		{
		for (colScan=0; colScan<4; colScan++) // porto a massa una colonna alla volta
			{
			PORTB=0xFF; // porto tutte le colonne a 1
			PORTB &= colMask[colScan]; // porto a zero la colonna attuale
			
			// mi controllo le righe
			for (rowScan=0; rowScan<4; rowScan++)
				{
				if (!(PORTB & rowMask[rowScan])) // Riga rowScan trovata a massa
					{
					DelayMs(50);
					if (!(PORTB & rowMask[rowScan]))
						{
						keypressed=rowScan+(4*colScan); // numero di pulsante premuto
						keyok=1; //  stato premuto un pulsante
						}
					}
				} // fine scansione righe
			} // fine scansione colonne
		
		if (keyok) //  stato premuto un pulsante
			{
			if (lcdXpos==MAX_X) // sono arrivato all'ultima colonna dell'LCD?
				{
				lcdXpos=0; // devo ritornare alla colonna zero
				lcdYpos++; // della riga successiva
				
				if (lcdYpos==MAX_Y) // sono finite le righe dell'LCD?
					{
					lcdYpos=0; // devo ritornare alla prima riga
					}
				
				LCD_GOTO(lcdYpos+1, lcdXpos+1); // posiziono il cursore sulla cella giusta
				}
			
			// solo dopo che ho controllato in che punto si trova il 
			// cursore posso piazzare il carattere premuto
			LCD_PUTCH(keys[keypressed]);
			sound();
			// incremento la colonna sull'LCD
			lcdXpos++;

			keyok=0; // resetto il flag di pulsante premuto
			
			// rimango in un ciclo continuo fino a che il pulsante non viene rilasciato
			PORTB=0b10101010;
			while(PORTB != 0b10101010)
				{continue;}
			}
		}
	}
	
void sound(void)
	{
	for (unsigned char a=10; a>0; a--)
		{
		DelayUs(200);
		BUZZER=1;
		DelayUs(200);
		BUZZER=0;
		}
	}